#!/usr/bin/perl
#############################################################################   
#
# decoy.pl -- no-ad redirection utility for squid or compatible cache apps
# (c) Vladi Belperchinov-Shabanski "Cade" 2002  
# <cade@biscom.net> <cade.datamax.bg> http://soul.datamax.bg/~cade
# $Id: decoy.pl,v 1.10 2002/12/12 07:10:13 cade Exp $
#
# DISTRIBUTED UNDER GPL LICENSE, SEE `COPYING' FILE FOR DETAILS
#
#############################################################################   
use strict;

### GLOBALS #################################################################   

our $CONFIG = find_config( "/etc/decoy.cfg", 
                           "/usr/etc/decoy.cfg", 
                           "/usr/local/etc/decoy.cfg", 
                           "/usr/local/decoy/decoy.cfg",
                           "decoy.cfg" );
our %CONFIG;
our @PATTERNS = ();
our $DEBUG = $ENV{ 'DEBUG' };

our $last_time;
our %last_time;
our $opt_reload = 1;

$| = 1;

$SIG{ HUP } = sub { $opt_reload = 1; };

my $run_time = time();

### MAIN ####################################################################   

reload();

while(<>)
  {
  check_files();
  reload() if $opt_reload;
  print check_line( $_ ) . "\n";
  }

$run_time = time() - $run_time;
logger( "STATUS: exit, run time: $run_time seconds" );

### END/SUBS BELOW ##########################################################   

sub check_files
{
  return if time() - $last_time < 60; # check files every 1 minute
  $last_time = time();
  
  for my $file ( ( $CONFIG, $CONFIG{ 'PATTERNS' } ) )
    {
    my $ct = file_mtime( $file );
    $opt_reload = 1 if $ct > $last_time{ $file };
    $last_time{ $file } = $ct;
    }
}

sub reload
{
  reload_config();
  reload_patterns();
  $opt_reload = 0;
}

sub reload_config
{
  print STDERR "CONFIG = $CONFIG\n" if $DEBUG;
  %CONFIG = load_hash( $CONFIG );
  while( my ( $k, $v ) = each %CONFIG )
    {
    print STDERR "CONFIG{ $k } = $v\n" if $DEBUG;
    }
}

sub reload_patterns
{
  $opt_reload = 0;
  @PATTERNS = ();
  my $i;
  open( $i, $CONFIG{ 'PATTERNS' } );
  while(<$i>)
    {
    s/[\s\r\n]+$//;
    s/^[\s\r\n]+//;
    next if /^#/;
    next unless /\S/;
    push @PATTERNS, $_;
    }
  close($i);
  my $cnt = @PATTERNS;
  logger( "INFO: $cnt patterns loaded" );
}

sub check_line
{
  my $line = $_[0];
  for my $re ( @PATTERNS )
    {
    if ( $line =~ /$re/i )
      {
      logger( "HIT: $line" );
      profile_log( $re );
      return $CONFIG{ 'REDIRECT' };
      }
    }
  return '';
}

### LIB #####################################################################   

sub file_mtime
{
  return (stat($_[0]))[9] or 0;
}

sub logger
{
  my $msg = $_[0];
  my $time = localtime time;
  print STDERR "$time: $msg\n" if $DEBUG;
  return unless $CONFIG{ 'LOG' };
  $msg =~ s/[\s\r\n]+$//;
  $msg =~ s/^[\s\r\n]//;
  my $o;
  open $o, ">>" . $CONFIG{ 'LOG' };
  print $o "$time [$$] $msg\n";
  close $o;
}

sub profile_log
{
  my $msg = $_[0];
  return unless $CONFIG{ 'PROFILE' };
  my $o;
  open $o, ">>" . $CONFIG{ 'PROFILE' };
  print $o "$msg\n";
  close $o;
}

sub find_config
{
  for( @_ )
    {
    return $_ if -e $_;
    }
  return undef;
}

sub save_array
{
  my $fn = shift;
  my $ar = shift;
  my $cnt = '0E0';
  my $o;
  open( o, '>' . $fn ) or return 0;
  for( @$ar )
    {
    s/[\r\n]+$//o;
    print $o $_ . "\n";
    $cnt++;
    }  
  close $o;
  return $cnt;
}

sub load_hash
{
  my $fn = shift;
  my %h;
  my $i;
  open( $i, $fn ) or return undef;
  while(<$i>)
    {
    next if /^\s*[;#]/;
    if ( /^\s*(\S+)\s*=(.*)$/ )
      {
      $h{ uc $1 } = $2;
      }
    }
  close $i;
  return %h;  
}

### DOC #####################################################################   

=pod

=head1 NAME

B<decoy.pl>  -- no-ad redirection utility for squid or compatible cache apps

=head1 DESCRIPTION

decoy.pl works as follows:

get line (url) from the standard input (squid) then check if this line can be
matched against list of regular expression patterns. if found return specific
url (usually empty gif image--1x1 transparent). if no match found it will
return empty line.

decoy.pl will reload configuration and patterns list if any of those two files
has been changed during decoy.pl's run time. reload will be done on HUP signal
as well. checks for file changes are performed on 1 minute (could be
configurable in the future).

decoy.pl is written for squid. yet it can be used in any cache or application
with the same behaviour (as squid).

=head1 FILES

Used files are:

=over *

=item decoy.pl

this is the main executable--can be placed anywhere you want.

=item decoy.cfg

this is configuration file which can be placed in one of these locations:

  /etc/decoy.cfg
  /usr/etc/decoy.cfg
  /usr/local/etc/decoy.cfg
  /usr/local/decoy/decoy.cfg

and finally in the current working directory (for test purposes mostly).

=item patterns.def

this file holds regular expressions (well mostly simple strings matching known
ad hosts). each line is considered expression. empty lines or lines with
leading `#' will be ignored.

this file name is not fixed and can be changed in the decoy.cfg

=item access.log

this file is used to log decoy.pl activity. its name is not fixed and can be
changed in decoy.cfg. (see CONFIGURATION)

=item profile.log

this file is used to log used patterns, i.e. every matched pattern will be 
logged for later statistics.

=back

=head1 CONFIGURATION

=head2 Example

  PATTERNS=/usr/local/decoy/patterns.def
  LOG=/usr/local/decoy/access.log
  PROFILE=/usr/local/decoy/profile.log
  REDIRECT=http://192.168.3.1/pub/banner.gif

=over *

=item PATTERNS

which file to be loaded as pattern definition.

=item LOG

which file to be used as activity log.

=item PROFILE

which file to be used as profile (pattern usage) log.

=item REDIRECT

url to which requests will be redirected if pattern matched.

=back

=head1 INSTALL

install decoy.pl anywhere you want. edit decoy.cfg as you want and set it in
one of the locations mentioned in the FILES/decoy.cfg section above. edit
squid.conf and set C<redirect_program> to point location of decoy.pl. for
example my is:

  redirect_program /usr/local/decoy/decoy.pl
  
restart squid

this should be enough, if you have problems contact me :)

=head1 LICENSE

GPL.

=head1 AUTHOR

  (c) Vladi Belperchinov-Shabanski "Cade" 2002  
  <cade@biscom.net> <cade.datamax.bg> http://soul.datamax.bg/~cade

=cut

### EOF #####################################################################   

